package de.unibi.techfak.scie.analysis;

import java.util.Collection;

import org.apache.uima.analysis_engine.AnalysisEngineProcessException;
import org.apache.uima.jcas.JCas;

import de.unibi.techfak.scie.annotators.AnnotationCounter;
import de.unibi.techfak.isy.lppba.descriptors.Aggregated;
import de.unibi.techfak.isy.lppba.descriptors.Drug;
import de.unibi.techfak.isy.lppba.descriptors.Error;
import de.unibi.techfak.isy.lppba.descriptors.Injury;
import de.unibi.techfak.isy.lppba.descriptors.InjuryType;
import de.unibi.techfak.isy.lppba.descriptors.InvestigationMethods;
import de.unibi.techfak.isy.lppba.descriptors.LabGroup;
import de.unibi.techfak.isy.lppba.descriptors.OntologyReference;
import de.unibi.techfak.isy.lppba.descriptors.Organism;
import de.unibi.techfak.isy.lppba.descriptors.Result;
import de.unibi.techfak.isy.lppba.descriptors.Treatment;
import de.unibi.techfak.scie.util.CachedJCasUtil;

public class AnalysisAnnotator extends org.apache.uima.fit.component.JCasAnnotator_ImplBase {

	//TODO:replace this after integration with the value that have been learned
	private static final double THRESHOLD = 0.2;
	//Every probability of a relation above THRESHOLD*DISTANCE is good
	private static final double DISTANCE = 1.1;
	//Relations to check
	private static final Class[] RELATIONS = {Treatment.class,Injury.class,Result.class,LabGroup.class};
	//Cores for Relations to check
	private static final Class[] CORES = { Drug.class,InjuryType.class,InvestigationMethods.class,Organism.class};
	
	@Override
	public void process(JCas jcas) throws AnalysisEngineProcessException {
		final CachedJCasUtil jcasUtil = CachedJCasUtil.getInstance(jcas);
		StringBuffer databases= new StringBuffer();
		StringBuffer relations= new StringBuffer();
		for(Class core:CORES){
			Collection<Object> typeCore=jcasUtil.select(core);
			if(typeCore.isEmpty()){
				String name=core.getName();
				String[] names=name.split("\\.");
				databases.append(names[names.length-1]+";");
			}
		}
		for(Class relation:RELATIONS){
			Collection<Aggregated> typeRelation=jcasUtil.select(relation);
			double maxProb=0;
			for(Aggregated agg:typeRelation){
				if(agg.getProbability()>maxProb){
					maxProb=agg.getProbability();
				}
			}
			if(maxProb<THRESHOLD*DISTANCE){
				String name=relation.getName();
				String[] names=name.split("\\.");
				relations.append(names[names.length-1]+";");
			}
		}
		Error error= new Error(jcas, 0, jcas.getDocumentText().length());
		error.setDatabases(databases.toString());
		error.setRelations(relations.toString());
		error.setAnnotationId(AnnotationCounter.getUniqueId());
		error.addToIndexes();
	}

}
