/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.techfak.scie.annotators.abbreviation;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.apache.uima.analysis_engine.AnalysisEngineProcessException;
import org.apache.uima.fit.util.JCasUtil;
import org.apache.uima.jcas.JCas;

import de.unibi.techfak.scie.annotators.ontology.OntologyAnnotator;
import de.unibi.techfak.scie.db.DatabaseSingleton;
import de.unibi.techfak.isy.lppba.descriptors.OntologyReference;
import de.unibi.techfak.isy.lppba.descriptors.Sentence;
import de.unibi.techfak.isy.lppba.descriptors.Token;
import de.unibi.techfak.scie.ontology.DictionaryMatch;
import de.unibi.techfak.scie.ontology.Node;
import de.unibi.techfak.scie.ontology.OntologyDatabase;
import de.unibi.techfak.scie.ontology.driver.NodeHandle;
import de.unibi.techfak.scie.ontology.driver.NodeName;
import de.unibi.techfak.scie.ontology.driver.OntologyHandle;
import de.unibi.techfak.scie.util.CachedJCasUtil;

public class AbbreviationAnnotator extends org.apache.uima.fit.component.JCasAnnotator_ImplBase {

	
	@Override
	public void process(JCas jcas) throws AnalysisEngineProcessException {
		HashMap<String, LinkedList<RefInfo>> abbreviation = new HashMap<String, LinkedList<RefInfo>>();
		final CachedJCasUtil jcasUtil = CachedJCasUtil.getInstance(jcas);
		final OntologyDatabase db = DatabaseSingleton.getInstance().
				getOntologyDatabase();
		Collection<OntologyReference> refs = jcasUtil.select(OntologyReference.class);
		for(OntologyReference ref:refs){
			Collection<Token> tokens=JCasUtil.selectFollowing(Token.class, ref, 1);
			if(!tokens.isEmpty()){
				Token positioning=tokens.iterator().next();
				if(positioning.getCoveredText().equals("(")){//Check for pattern "longform(shortform)"
					StringBuffer shortform= new StringBuffer();
					//shortform in parenthesis is assumed to be max 2 words
					tokens=JCasUtil.selectFollowing(Token.class, positioning, 3);
					boolean isShortCandidate=false;
					for(Token token:tokens){
						if(token.getCoveredText().equals(")")){
							isShortCandidate=true;
							break;
						}else{
								shortform.append(token.getCoveredText()+" ");
						}
					}
					if(isShortCandidate){
						String shortCandidate=shortform.toString().trim();
						OntologyHandle onto=db.getOntology(ref.getOntology());
						NodeHandle handle =new NodeHandle(onto, ref.getId());
						Node node=db.getNode(handle);
						for(NodeName name : node.getNames()){
							//if(checkShortForm(shortCandidate, name.getName())){
							if(name.getName().equals(findBestLongForm(shortCandidate, name.getName()))){
								//addAbbreviation
								LinkedList<RefInfo> abbreviationList;
								if(!abbreviation.containsKey(shortform)){
									 abbreviationList= new LinkedList<RefInfo>();
									 abbreviation.put(shortCandidate, abbreviationList);
								}else{
									 abbreviationList= abbreviation.get(shortCandidate);
								}
								abbreviationList.add(new RefInfo(name,handle));
//								System.out.println("found pattern longform(shortform)");
//								System.out.println("Kurz: \""+shortCandidate+"\""+" Lang: \""+name.getName()+"\"");
//								System.out.println("Inerhalb von:");
//								System.out.println(jcas.getDocumentText().substring(ref.getBegin()-50, ref.getEnd()+50));
								break;
							}
						}
						
					}
				}else if(positioning.equals(")")){//Check for pattern "shortform(longform)"
					StringBuffer shortform= new StringBuffer();
					//shortform in parenthesis is assumed to be max 2 words
					tokens=JCasUtil.selectPreceding(Token.class, positioning, 3);
					boolean isShortCandidate=false;
					for(Token token:tokens){
						if(token.getCoveredText().equals("(")){
							isShortCandidate=true;
						}else{
							if(isShortCandidate){
								shortform.append(token.getCoveredText()+" ");								
							}
						}
					}
					if(isShortCandidate){
						String shortCandidate=shortform.toString().trim();
						OntologyHandle onto=db.getOntology(ref.getOntology());
						NodeHandle handle =new NodeHandle(onto, ref.getId());
						Node node=db.getNode(handle);
						for(NodeName name : node.getNames()){
							if(name.getName().equals(findBestLongForm(shortCandidate, name.getName()))){
								//addAbbreviation
								LinkedList<RefInfo> abbreviationList;
								if(!abbreviation.containsKey(shortform)){
									 abbreviationList= new LinkedList<RefInfo>();
									 abbreviation.put(shortCandidate, abbreviationList);
								}else{
									 abbreviationList= abbreviation.get(shortCandidate);
								}
								abbreviationList.add(new RefInfo(name,handle));
//								System.out.println("found pattern shortform(longform)");
//								System.out.println("Kurz: \""+shortCandidate+"\""+" Lang: \""+name.getName()+"\"");
//								System.out.println("Inerhalb von:");
//								System.out.println(jcas.getDocumentText().substring(ref.getBegin()-50, ref.getEnd()+50));
								break;
							}
						}
					}
				}
			}
			
			
		}
		searchAbbreviation(abbreviation, jcas);
	}
	
	private void searchAbbreviation(HashMap<String, LinkedList<RefInfo>> abbreviation, JCas jcas){
		final CachedJCasUtil jcasUtil = CachedJCasUtil.getInstance(jcas);
		Collection<Token> tokens = jcasUtil.select(Token.class);
		for(Token token:tokens){
		 for (Map.Entry<String, LinkedList<RefInfo>> entry : abbreviation.entrySet()){
			 if(token.getCoveredText().equals(entry.getKey())){
				 for(RefInfo tag : entry.getValue()){
					 OntologyAnnotator.createOntologyReferenceAnnotation(jcas, new DictionaryMatch(tag.handle, tag.name, token.getBegin(), token.getEnd(), 1));
				 }
			 }else{
				 List<Token> tokens2=JCasUtil.selectFollowing(Token.class, token, 1);
				 if(tokens2.size()==0){
					 continue;
				 }
				 Token token2=tokens2.get(0);
				 if(token.getCoveredText().concat(token2.getCoveredText()).equals(entry.getKey())){
					 for(RefInfo tag : entry.getValue()){
						 OntologyAnnotator.createOntologyReferenceAnnotation(jcas, new DictionaryMatch(tag.handle, tag.name, token.getBegin(), token2.getEnd(), 1));
					 }
				 }
			 }
		 }
		}
	}
	
	 
	
	//similar to findBestLongForm from Ariel S. Schwartz and Marti A. Hearst. 2003.
	//A simple algorithm for identifying abbreviation definitions in biomedical text.
	//Pac Symp Biocomput, pages 451–462
	private boolean checkShortForm(String shortForm, String longForm) {
		int length =shortForm.length();
		if(length<2||length>10){
			return false;
		}
		int sIndex;
		// The index on the short form
		int lIndex;
		// The index on the long form
		char currChar; // The current character to match
		sIndex = shortForm.length() - 1;
		// Set sIndex at the end of the
		// short form
		lIndex = longForm.length() - 1;
		// Set lIndex at the end of the
		// long form
		for ( ; sIndex >= 0; sIndex--) { // Scan the short form starting
		// from end to start
		// Store the next character to match. Ignore case
		currChar = Character.toLowerCase(shortForm.charAt(sIndex));
		// ignore non alphanumeric characters
		if (!Character.isLetterOrDigit(currChar))
		continue;
		// Decrease lIndex while current character in the long form
		// does not match the current character in the short form.
		// If the current character is the first character in the
		// short form, decrement lIndex until a matching character
		// is found at the beginning of a word in the long form.
		while (
		((lIndex >= 0) &&
		(Character.toLowerCase(longForm.charAt(lIndex)) != currChar))
		||
		((sIndex == 0) && (lIndex > 0) &&
		(Character.isLetterOrDigit(longForm.charAt(lIndex - 1)))))
		lIndex--;
		// If no match was found in the long form for the current
		// character, return null (no match).
		if (lIndex < 0)
		return false;
		// A match was found for the current character. Move to the
		// next character in the long form.
		lIndex--;
		}
		// Find the beginning of the first word (in case the first
		// character matches the beginning of a hyphenated word).
		lIndex = longForm.lastIndexOf(" ", lIndex) + 1;
		// Return the best long form, the substring of the original
		// long form, starting from lIndex up to the end of the original
		// long form.
		return !longForm.substring(lIndex).equals("");
	}
	
	private String findBestLongForm(String shortForm, String longForm) {
		int sIndex;
		// The index on the short form
		int lIndex;
		// The index on the long form
		char currChar; // The current character to match
		sIndex = shortForm.length() - 1;
		// Set sIndex at the end of the
		// short form
		lIndex = longForm.length() - 1;
		// Set lIndex at the end of the
		// long form
		for ( ; sIndex >= 0; sIndex--) { // Scan the short form starting
		// from end to start
		// Store the next character to match. Ignore case
		currChar = Character.toLowerCase(shortForm.charAt(sIndex));
		// ignore non alphanumeric characters
		if (!Character.isLetterOrDigit(currChar))
		continue;
		// Decrease lIndex while current character in the long form
		// does not match the current character in the short form.
		// If the current character is the first character in the
		// short form, decrement lIndex until a matching character
		// is found at the beginning of a word in the long form.
		while (
		((lIndex >= 0) &&
		(Character.toLowerCase(longForm.charAt(lIndex)) != currChar))
		||
		((sIndex == 0) && (lIndex > 0) &&
		(Character.isLetterOrDigit(longForm.charAt(lIndex - 1)))))
		lIndex--;
		// If no match was found in the long form for the current
		// character, return null (no match).
		if (lIndex < 0)
		return null;
		// A match was found for the current character. Move to the
		// next character in the long form.
		lIndex--;
		}
		// Find the beginning of the first word (in case the first
		// character matches the beginning of a hyphenated word).
		lIndex = longForm.lastIndexOf(" ", lIndex) + 1;
		// Return the best long form, the substring of the original
		// long form, starting from lIndex up to the end of the original
		// long form
		return longForm.substring(lIndex);
	}
}
