/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.techfak.scie.annotators.quantities;

import de.unibi.techfak.scie.annotators.AnnotationCounter;
import java.util.Collection;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.uima.analysis_engine.AnalysisEngineProcessException;
import org.apache.uima.fit.component.JCasAnnotator_ImplBase;
import org.apache.uima.fit.util.JCasUtil;
import org.apache.uima.jcas.JCas;

import de.unibi.techfak.isy.lppba.descriptors.Duration;
import de.unibi.techfak.isy.lppba.descriptors.Time;
import de.unibi.techfak.isy.lppba.descriptors.Token;

public class DurationAnnotator extends JCasAnnotator_ImplBase {

	private static final Pattern DURATION = Pattern.compile("(for)|(over)");

	@Override
	public void process(JCas jcas) throws AnalysisEngineProcessException {
		Collection<Time> times = JCasUtil.select(jcas, Time.class);
		for (final Time time : times) {
			final List<Token> prev = JCasUtil.selectPreceding(Token.class, time, 1);
			for (final Token t : prev) {
				final Matcher matcher = DURATION.matcher(t.getCoveredText()
						.toLowerCase());
				if (matcher.matches()) {
					Duration duration = new Duration(jcas, t.getBegin(), time.getEnd());
					duration.setTime(time);
					duration.setAnnotationId(AnnotationCounter.getUniqueId());
					duration.addToIndexes();
				}
			}
		}

	}

}
