/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.techfak.scie.annotators.quantities;

import de.unibi.techfak.scie.annotators.AnnotationCounter;
import java.util.Collection;
import org.apache.uima.analysis_engine.AnalysisEngineProcessException;
import org.apache.uima.fit.util.JCasUtil;
import org.apache.uima.jcas.JCas;

import de.unibi.techfak.isy.lppba.descriptors.WordNumber;

/**
 * This transforms WordNumberAnnotations as done by the OntologyAnnotator into Integer annotations.
 *
 * @author Benjamin Paassen - bpaassen@techfak.uni-bielefeld.de
 */
public class WordNumberAnnotator
		extends org.apache.uima.fit.component.JCasAnnotator_ImplBase {

	public WordNumberAnnotator() {
	}

	@Override
	public void process(JCas jcas) throws AnalysisEngineProcessException {
		final Collection<WordNumber> wordNumbers = JCasUtil.select(jcas, WordNumber.class);
		for (final WordNumber wn : wordNumbers) {
			final de.unibi.techfak.isy.lppba.descriptors.Integer i =
					new de.unibi.techfak.isy.lppba.descriptors.Integer(jcas, wn.getBegin(), wn.
					getEnd());
			i.setValue(wn.getValue());
			i.setAnnotationId(AnnotationCounter.getUniqueId());
			i.addToIndexes();
		}
	}
}
