/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.techfak.scie.annotators.result;

import de.unibi.techfak.scie.annotators.AnnotationCounter;
import java.util.Collection;

import org.apache.uima.analysis_engine.AnalysisEngineProcessException;
import org.apache.uima.fit.component.JCasAnnotator_ImplBase;
import org.apache.uima.fit.util.JCasUtil;
import org.apache.uima.jcas.JCas;

import de.unibi.techfak.isy.lppba.descriptors.Judgement;
import de.unibi.techfak.isy.lppba.descriptors.Trend;

public class JudgementAnnotator extends JCasAnnotator_ImplBase {

	@Override
	public void process(JCas jcas) throws AnalysisEngineProcessException {
		Collection<Trend> trends = JCasUtil.select(jcas, Trend.class);
		for (final Trend trend : trends) {
			Judgement judg = new Judgement(jcas, trend.getBegin(), trend.getEnd());
			switch (trend.getNormalizedName()) {
			case "improvement":
				judg.setIsPositiv(true);
				judg.setIsNegative(false);
				judg.setIsNeutral(false);
				break;
			case "increase":
				judg.setIsPositiv(true);
				judg.setIsNegative(false);
				judg.setIsNeutral(false);
				break;
			case "decrease":
				judg.setIsPositiv(false);
				judg.setIsNegative(true);
				judg.setIsNeutral(false);
				break;
			case "difference":
				judg.setIsPositiv(false);
				judg.setIsNegative(false);
				judg.setIsNeutral(true);
				break;
			case "no difference":
				judg.setIsPositiv(false);
				judg.setIsNegative(false);
				judg.setIsNeutral(true);
				break;
			default:
				judg.setIsPositiv(false);
				judg.setIsNegative(false);
				judg.setIsNeutral(true);
				break;
			}
			judg.setAnnotationId(AnnotationCounter.getUniqueId());
			judg.addToIndexes();
		}

	}

}
