/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.techfak.scie.annotators.structure;

import java.util.Collection;
import java.util.List;

import org.apache.uima.analysis_engine.AnalysisEngineProcessException;
import org.apache.uima.fit.util.JCasUtil;
import org.apache.uima.jcas.JCas;

import de.unibi.techfak.isy.lppba.descriptors.Header;
import de.unibi.techfak.isy.lppba.descriptors.Section;
import de.unibi.techfak.isy.lppba.descriptors.TextBlock;
import de.unibi.techfak.scie.util.CachedJCasUtil;

/**
 * This annotates sections in the text. A section is defined here as a header
 * plus every text until the next header.
 *
 * @author Benjamin Paassen - bpaassen@techfak.uni-bielefeld.de
 */
public class SectionAnnotator extends
	org.apache.uima.fit.component.JCasAnnotator_ImplBase {

	@Override
	public void process(JCas jcas) throws AnalysisEngineProcessException {
		final Collection<Header> headers = JCasUtil.select(jcas, Header.class);
		final CachedJCasUtil jcasUtil = CachedJCasUtil.getInstance(jcas);
		Header lastHeader = null;
		for (final Header header : headers) {
			if (lastHeader != null) {
				// get all TextBlocks between the last header and this one.
				final List<TextBlock> blocks = jcasUtil.selectCovered(TextBlock.class,
					lastHeader.getEnd(), header.getBegin());
				// we only make a section if we have at least one such block.
				if (!blocks.isEmpty()) {
					final Section section = new Section(jcas, lastHeader.getBegin(),
						blocks.get(blocks.size() - 1).getEnd());
					section.setHeader(lastHeader);
					section.addToIndexes();
				}
			}
			lastHeader = header;
		}
		if (lastHeader != null) {
			// insert the last section.
			int textEnd = jcas.getDocumentText().length();
			final Section lastSection = new Section(jcas, lastHeader.getBegin(),
				textEnd);
			lastSection.setHeader(lastHeader);
			lastSection.addToIndexes();
		}
	}
}
