/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.techfak.scie.annotators.treatment;

import de.unibi.techfak.scie.annotators.AnnotationCounter;
import de.unibi.techfak.isy.lppba.descriptors.AntiBody;
import de.unibi.techfak.isy.lppba.descriptors.Token;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import org.apache.uima.analysis_engine.AnalysisEngineProcessException;
import org.apache.uima.fit.component.JCasAnnotator_ImplBase;
import org.apache.uima.fit.util.JCasUtil;
import org.apache.uima.jcas.JCas;

public class AntiBodyAnnotator extends JCasAnnotator_ImplBase {
	private static final String ANTI = "anti-cd";
	private static final String MAB = "mab";

	@Override
	public void process(JCas jcas) throws AnalysisEngineProcessException {
		Collection<Token> words = JCasUtil.select(jcas, Token.class);
		Iterator<Token> iterator = words.iterator();
		if (iterator.hasNext()) {
			for (Token start = iterator.next(); iterator.hasNext(); start = iterator
					.next()) {
				if (start.getCoveredText().toLowerCase().startsWith(ANTI)) {
					List<Token> wordlist = JCasUtil.selectFollowing(Token.class, start, 1);
					Token end = wordlist.get(wordlist.size() - 1);
					if (end.getCoveredText().toLowerCase().startsWith(MAB)) {
						AntiBody antiBody = new AntiBody(jcas);
						antiBody.setBegin(start.getBegin());
						antiBody.setEnd(end.getEnd());
						antiBody.setAnnotationId(AnnotationCounter.getUniqueId());
						antiBody.addToIndexes();
					}
				}
			}
		}
	}
}
