/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.techfak.scie.annotators.treatment;

import de.unibi.techfak.scie.annotators.AnnotationCounter;
import java.util.List;

import org.apache.uima.jcas.JCas;

import de.unibi.techfak.scie.annotators.quantities.Unit;
import de.unibi.techfak.scie.annotators.quantities.UnitQuantityAnnotator;
import de.unibi.techfak.isy.lppba.descriptors.Dose;
import de.unibi.techfak.isy.lppba.descriptors.Quantity;
import de.unibi.techfak.isy.lppba.descriptors.Token;

/**
 * This annotates Doses
 * 
 * @author Raphael
 * 
 */
public class DoseAnnotator extends UnitQuantityAnnotator {
	private static final Unit[] units = { Unit.IUPERKG, Unit.MILLIGRAMMESPERKG,Unit.GRAMMESPERKG ,Unit.MILLIGRAMMESPERKGPERHOUR, Unit.GRAMMESPERKGPERHOUR};

	public DoseAnnotator() {
		super(units, 1);
	}

	@Override
	public void handleQuantity(Quantity q, Unit unit, List<Token> unitTokens,
			JCas jcas) {
		Token unitToken = unitTokens.get(unitTokens.size() - 1);
		// then create the annotation.
		final int begin = q.getBegin();
		final int end = unitToken.getEnd();
		final Dose dose = new Dose(jcas, begin, end);
		dose.setUnitName(unit.getNormalizedName());
		dose.setValue(q);
		dose.setSIValue(-1);
		dose.setAnnotationId(AnnotationCounter.getUniqueId());
		dose.addToIndexes();

	}

}
