/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.techfak.scie.configuration;

/**
 *
 * @author Andreas Stoeckel -- astoecke(at)techfak.uni-bielefeld.de
 */
public class ConfigurationEntry {

	public interface DefaultProvider {

		public String getDefaultValue();

	}

	public interface ValidityChecker {

		public Boolean isValid(String value) throws ValidityException;

	}

	public enum ConfiguationEntryType {

		typeString, typeNumber;
	}

	public static ValidityChecker ALWAYS_TRUE = new ValidityChecker() {
		@Override
		public Boolean isValid(String value) throws ValidityException {
			return true;
		}
	};

	private final String key;

	private final String description;

	private final DefaultProvider defaultProvider;

	private final ValidityChecker validityChecker;

	public ConfigurationEntry(String key, String description,
			DefaultProvider defaultProvider, ValidityChecker validityChecker) {
		this.key = key.toLowerCase();
		this.description = description;
		this.defaultProvider = defaultProvider;
		this.validityChecker = validityChecker;
	}

	public ConfigurationEntry(String key, String description,
			final String defaultValue, ValidityChecker validityChecker) {
		this(key, description, new DefaultProvider() {
			@Override
			public String getDefaultValue() {
				return defaultValue;
			}
		}, validityChecker);
	}

	public ConfigurationEntry(String key, String description,
			String defaultValue) {
		this(key, description, defaultValue, ALWAYS_TRUE);
	}

	public ConfigurationEntry(String key, String description,
			DefaultProvider defaultProvider) {
		this(key, description, defaultProvider, ALWAYS_TRUE);
	}

	public String getKey() {
		return key;
	}

	public String getDescription() {
		return description;
	}

	public String getDefault() {
		return defaultProvider.getDefaultValue();
	}

	public boolean isValid(String value) throws ValidityException {
		return validityChecker.isValid(value);
	}

}
