/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.techfak.scie.document_import;

import java.io.InputStream;
import java.io.IOException;
import org.apache.uima.jcas.JCas;

/**
 * The DocumentImporter class describes a very simple interface that has to be
 * implemented by every component that is capable of parsing document content
 * and transforming it into a UIMA CAS (Common Analysis Structure) using the
 * annotations defined in this package.
 *
 * @author astoecke
 */
public abstract class DocumentImporter {

	/**
	 * The input stream given in the constructor. Implementations of
	 * DocumentImporter should read the source document from this stream.
	 */
	protected InputStream input;

	/**
	 * The target CAS instance. The content of the document as well as all
	 * annotations related to the source document have to be written to this
	 * Common Analysis Structure.
	 */
	protected JCas target;

	/**
	 * Creates the instance of DocumentImporter.
	 * 
	 * @param input is the input stream from which the document should be read.
	 * @param target is the target UIMA Common Analysis Structure to which
	 * the document contents are written and annotated.
	 */
	public DocumentImporter(InputStream input, JCas target) {
		this.input = input;
		this.target = target;
	}

	/**
	 * Imports the document provided by the input stream and writes the
         * contents to the given CAS instance.
	 *
	 * @throws IOException if an error occurs while reading from the input
	 * stream.
	 */
	public abstract void doImport() throws IOException;

}
