/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.techfak.scie.probability_model;

/**
 * This is a wrapper class to return a hypothesis with its probability to belong
 * to an Annotation Composition.
 *
 * @author Benjamin Paassen - bpaassen@techfak.uni-bielefeld.de
 */
public class ProbableHypothesis<X extends AnnotationCandidate> implements
		Comparable<ProbableHypothesis> {

	private X candidate;
	private double probability;

	public ProbableHypothesis() {
	}

	public ProbableHypothesis(X candidate, double posterior) {
		this.candidate = candidate;
		this.probability = posterior;
	}

	/**
	 * Get the value of candidate
	 *
	 * @return the value of candidate
	 */
	public X getCandidate() {
		return candidate;
	}

	/**
	 * Set the value of candidate
	 *
	 * @param candidate new value of candidate
	 */
	public void setCandidate(X candidate) {
		this.candidate = candidate;
	}

	/**
	 * Returns the probability that this candidate belongs to an Annotation
	 * compositon. The probability is calculated as the semantic probability of
	 * this annotation belonging to the compositon multiplied with the syntactic
	 * probability thereof.
	 *
	 * @return the probability that this candidate belongs to an Annotation
	 * compositon.
	 */
	public double getProbability() {
		return probability;
	}

	/**
	 *
	 * @param probability the probability that this candidate belongs to an
	 * Annotation compositon. The probability is calculated as the semantic
	 * probability of this annotation belonging to the compositon multiplied
	 * with the syntactic probability thereof.
	 */
	public void setProbability(double probability) {
		this.probability = probability;
	}

	@Override
	public int compareTo(ProbableHypothesis o) {
		if (o == null) {
			return 1;
		}
		return Double.compare(probability, o.probability);
	}
}
