/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.techfak.scie.annotators.animal;

import de.unibi.techfak.scie.annotators.animal.ExactAgeAnnotator;
import de.unibi.techfak.scie.annotators.ontology.OntologyAnnotator;
import de.unibi.techfak.scie.annotators.quantities.NumberAnnotator;
import de.unibi.techfak.scie.annotators.quantities.TimeAnnotator;
import de.unibi.techfak.scie.annotators.quantities.WordNumberAnnotator;
import de.unibi.techfak.scie.annotators.structure.StanfordTokenizer;
import de.unibi.techfak.scie.db.DatabaseSingleton;
import de.unibi.techfak.isy.lppba.descriptors.ExactAge;
import java.util.Collection;
import java.util.HashSet;
import static junit.framework.Assert.assertTrue;
import junit.framework.TestCase;
import org.apache.uima.fit.factory.JCasFactory;
import org.apache.uima.fit.util.JCasUtil;
import org.apache.uima.jcas.JCas;

/**
 *
 * @author Benjamin Paassen - bpaassen(at)techfak.uni-bielefeld.de
 */
public class ExactAgeAnnotatorTest extends TestCase {

	public ExactAgeAnnotatorTest(String testName) {
		super(testName);
	}

	@Override
	protected void setUp() throws Exception {
		super.setUp();
		DatabaseSingleton.initialize("./config.xml");
	}

	@Override
	protected void tearDown() throws Exception {
		super.tearDown();
	}

	/**
	 * Test of process method, of class ExactAgeAnnotator.
	 */
	public void testProcess() throws Exception {

		//create example
		final JCas jcas = JCasFactory.createJCas(
				"src/main/resources/de/unibi/techfak/isy/lppba/descriptors/LPPBATypeSystem");
		jcas.setDocumentText("A 10 years old man. A woman of one days age. A 1,3 h test.");
		//preprosess using StanfordTokenizer, NumberAnnotator, WordNumberAnnotator and TimeAnnotator
		final StanfordTokenizer st = new StanfordTokenizer();
		st.process(jcas);
		final NumberAnnotator numberAnnotator = new NumberAnnotator();
		numberAnnotator.process(jcas);
		final OntologyAnnotator ontoAnnotator = new OntologyAnnotator();
		ontoAnnotator.process(jcas);
		final WordNumberAnnotator wordNumberAnnotator = new WordNumberAnnotator();
		wordNumberAnnotator.process(jcas);
		final TimeAnnotator timeAnnotator = new TimeAnnotator();
		timeAnnotator.process(jcas);

		//use the ExactAgeAnnotator
		final ExactAgeAnnotator instance = new ExactAgeAnnotator();
		instance.process(jcas);

		//check results
		final Collection<ExactAge> instances = JCasUtil.select(jcas, ExactAge.class);

		final String[] expected = {"10 years old", "one days age"};
		final HashSet<String> actual = new HashSet<>();
		for (final ExactAge age : instances) {
			actual.add(age.getCoveredText());
		}
		for (final String exp : expected) {
			assertTrue(exp + " was not found!", actual.contains(exp));
		}
	}
}
