/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.techfak.scie.annotators.animal;

import de.unibi.techfak.scie.annotators.ontology.OntologyAnnotator;
import de.unibi.techfak.scie.db.DatabaseSingleton;
import de.unibi.techfak.isy.lppba.descriptors.Gender;

import java.util.Collection;
import java.util.Iterator;

import junit.framework.TestCase;

import org.apache.uima.fit.factory.JCasFactory;
import org.apache.uima.fit.util.JCasUtil;
import org.apache.uima.jcas.JCas;

/**
 *
 * @author Benjamin Paassen - bpaassen(at)techfak.uni-bielefeld.de
 */
public class GenderAnnotatorTest extends TestCase {

	public GenderAnnotatorTest(String testName) {
		super(testName);
	}

	@Override
	protected void setUp() throws Exception {
		super.setUp();
	}

	@Override
	protected void tearDown() throws Exception {
		super.tearDown();
	}

	/**
	 * Test of process method, of class GenderAnnotator.
	 */
	public void testProcess() throws Exception {
		DatabaseSingleton.initialize("./config.xml");

		//create example
		final JCas jcas = JCasFactory.createJCas(
			"src/main/resources/de/unibi/techfak/isy/lppba/descriptors/LPPBATypeSystem");
		jcas.setDocumentText(" female male malefitz.");

		//use the GenderAnnotator
		final OntologyAnnotator instance = new OntologyAnnotator();
		instance.process(jcas);

		//check results
		final Collection<Gender> genderInstances = JCasUtil.select(jcas, Gender.class);
		assertEquals(2, genderInstances.size());
		final Iterator<Gender> iterator = genderInstances.iterator();
		Gender gender = iterator.next();
		assertEquals(1, gender.getBegin());
		assertEquals(7, gender.getEnd());
		assertEquals("female", gender.getCoveredText());
		gender = iterator.next();
		assertEquals(8, gender.getBegin());
		assertEquals(12, gender.getEnd());
		assertEquals("male", gender.getCoveredText());
	}
}
