/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.techfak.scie.annotators.injury;

import de.unibi.techfak.scie.annotators.injury.VertebralPositionAnnotator;
import de.unibi.techfak.scie.annotators.quantities.NumberAnnotator;
import de.unibi.techfak.scie.annotators.quantities.RangeAnnotator;
import de.unibi.techfak.scie.annotators.structure.StanfordTokenizer;
import de.unibi.techfak.isy.lppba.descriptors.Range;
import de.unibi.techfak.isy.lppba.descriptors.VertebralPosition;
import java.util.Collection;
import java.util.Iterator;
import static junit.framework.Assert.assertEquals;
import junit.framework.TestCase;
import org.apache.uima.fit.factory.JCasFactory;
import org.apache.uima.fit.util.JCasUtil;
import org.apache.uima.jcas.JCas;

/**
 *
 * @author socnow
 */
public class VertebralPositionAnnotatorTest extends TestCase {

	public VertebralPositionAnnotatorTest(String testName) {
		super(testName);
	}

	@Override
	protected void setUp() throws Exception {
		super.setUp();
	}

	@Override
	protected void tearDown() throws Exception {
		super.tearDown();
	}

	/**
	 * Test of process method, of class VertebralPositionAnnotator.
	 */
	public void testProcess() throws Exception {
		//create example
		final JCas jcas = JCasFactory.createJCas(
				"src/main/resources/de/unibi/techfak/isy/lppba/descriptors/LPPBATypeSystem");
		jcas.setDocumentText(
				"Prefix. T9-10 vertebral level. C3 laminectomy.");
		//preprosess using StanfordTokenizer, NumberAnnotator and RangeAnnotator
		final StanfordTokenizer st = new StanfordTokenizer();
		st.process(jcas);
		final NumberAnnotator na = new NumberAnnotator();
		na.process(jcas);
		final RangeAnnotator ra = new RangeAnnotator();
		ra.process(jcas);

		//use the VertebralPositionAnnotator
		final VertebralPositionAnnotator instance =
				new VertebralPositionAnnotator();
		instance.process(jcas);

		//check results
		final Collection<VertebralPosition> vps = JCasUtil.select(jcas,
				VertebralPosition.class);

		final Iterator<VertebralPosition> iterator = vps.iterator();

		assertTrue(iterator.hasNext());
		VertebralPosition vp = iterator.next();
		assertEquals("T9-10", vp.getCoveredText());
		assertEquals("Thoracic",vp.getRegion());
		assertTrue(vp.getNumber() instanceof Range);
		assertTrue(
				((Range) vp.getNumber()).getLowVal() instanceof de.unibi.techfak.isy.lppba.descriptors.Integer);
		final int lowVal =
				((de.unibi.techfak.isy.lppba.descriptors.Integer) ((Range) vp.
				getNumber()).getLowVal()).getValue();
		final int highVal =
				((de.unibi.techfak.isy.lppba.descriptors.Integer) ((Range) vp.
				getNumber()).getHighVal()).getValue();
		assertEquals(9, lowVal);
		assertEquals(10, highVal);

		assertTrue(iterator.hasNext());
		vp = iterator.next();
		assertEquals("C3", vp.getCoveredText());
		assertEquals("Cervical",vp.getRegion());
		assertTrue(
				vp.getNumber() instanceof de.unibi.techfak.isy.lppba.descriptors.Integer);
		final int val =
				((de.unibi.techfak.isy.lppba.descriptors.Integer) vp.
				getNumber()).getValue();

		assertEquals(3, val);
	}
}
