/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.techfak.scie.annotators.quantities;

import de.unibi.techfak.scie.annotators.quantities.DurationAnnotator;
import de.unibi.techfak.scie.annotators.quantities.NumberAnnotator;
import de.unibi.techfak.scie.annotators.quantities.WordNumberAnnotator;
import de.unibi.techfak.scie.annotators.quantities.TimeAnnotator;
import de.unibi.techfak.scie.annotators.ontology.OntologyAnnotator;
import de.unibi.techfak.scie.annotators.structure.StanfordTokenizer;
import java.util.Collection;

import org.apache.uima.fit.factory.JCasFactory;
import org.apache.uima.fit.util.JCasUtil;
import org.apache.uima.jcas.JCas;

import de.unibi.techfak.scie.db.DatabaseSingleton;
import de.unibi.techfak.isy.lppba.descriptors.Duration;
import java.util.HashSet;
import junit.framework.TestCase;

public class DurationAnnotatorTest extends TestCase {

	public DurationAnnotatorTest(String testName) {
		super(testName);
	}

	@Override
	protected void setUp() throws Exception {
		super.setUp();
		DatabaseSingleton.initialize("./config.xml");
	}

	@Override
	protected void tearDown() throws Exception {
		super.tearDown();
	}

	/**
	 * Test of process method, of class DurationAnnotator.
	 */
	public void testProcess() throws Exception {

		//create example
		final JCas jcas = JCasFactory.createJCas(
				"src/main/resources/de/unibi/techfak/isy/lppba/descriptors/LPPBATypeSystem");
		jcas.setDocumentText(
				"for 10 years an old man eat honey. For one day an old man was lucky. For 1,3 h tested. over 1,3 h treatment");
		//preprosess using StanfordTokenizer, NumberAnnotator and WordNumberAnnotator
		final StanfordTokenizer st = new StanfordTokenizer();
		st.process(jcas);
		final NumberAnnotator numberAnnotator = new NumberAnnotator();
		numberAnnotator.process(jcas);
		final OntologyAnnotator ontoAnnotator = new OntologyAnnotator();
		ontoAnnotator.process(jcas);
		final WordNumberAnnotator wordNumberAnnotator = new WordNumberAnnotator();
		wordNumberAnnotator.process(jcas);

		//use the TimeAnnotator
		final TimeAnnotator timeAnnotator = new TimeAnnotator();
		timeAnnotator.process(jcas);
		//use the Duration annotator
		final DurationAnnotator instance = new DurationAnnotator();
		instance.process(jcas);
		//check results
		final Collection<Duration> durationInstances = JCasUtil.select(jcas, Duration.class);

		final String[] expected = {"for 10 years", "For one day", "For 1,3 h", "over 1,3 h"};

		final HashSet<String> actual = new HashSet<>();
		for (final Duration d : durationInstances) {
			actual.add(d.getCoveredText());
		}
		for (final String exp : expected) {
			assertTrue(exp + " was expected but not found!", actual.contains(exp));
		}
	}
}
