/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.techfak.scie.annotators.quantities;

import de.unibi.techfak.scie.annotators.quantities.NumberAnnotator;
import de.unibi.techfak.scie.annotators.quantities.RangeAnnotator;
import java.util.Collection;
import java.util.Iterator;

import junit.framework.TestCase;

import org.apache.uima.fit.factory.JCasFactory;
import org.apache.uima.fit.util.JCasUtil;
import org.apache.uima.jcas.JCas;

import de.unibi.techfak.isy.lppba.descriptors.Range;
import static junit.framework.Assert.assertEquals;
import static junit.framework.Assert.assertTrue;

public class RangeAnnotatorTest extends TestCase {

	public RangeAnnotatorTest(String testName) {
		super(testName);
	}

	@Override
	protected void setUp() throws Exception {
		super.setUp();
	}

	@Override
	protected void tearDown() throws Exception {
		super.tearDown();
	}

	/**
	 * Test of process method, of class RangeAnnotator.
	 */
	public void testProcess() throws Exception {
		// create example
		final JCas jcas = JCasFactory
				.createJCas(
				"src/main/resources/de/unibi/techfak/isy/lppba/descriptors/LPPBATypeSystem");
		jcas.setDocumentText("1-2 weeks old, 3 to 4 years. 2.1- 3.5!");
		final NumberAnnotator numbersAnnotator = new NumberAnnotator();
		numbersAnnotator.process(jcas);

		// use the RangeAnnotator
		final RangeAnnotator instance = new RangeAnnotator();
		instance.process(jcas);

		// check results
		final Collection<Range> rangeInstances = JCasUtil
				.select(jcas, Range.class);

		final Iterator<Range> iterator = rangeInstances.iterator();
		assertTrue(iterator.hasNext());
		Range range = iterator.next();
		assertEquals("1-2", range.getCoveredText());
		de.unibi.techfak.isy.lppba.descriptors.Number lowVal = range.getLowVal();
		de.unibi.techfak.isy.lppba.descriptors.Number highVal = range.
				getHighVal();
		assertTrue(
				lowVal instanceof de.unibi.techfak.isy.lppba.descriptors.Integer);
		assertTrue(
				highVal instanceof de.unibi.techfak.isy.lppba.descriptors.Integer);
		assertEquals(1,
				((de.unibi.techfak.isy.lppba.descriptors.Integer) lowVal).
				getValue());
		assertEquals(2,
				((de.unibi.techfak.isy.lppba.descriptors.Integer) highVal).
				getValue());

		assertTrue(iterator.hasNext());
		range = iterator.next();
		assertEquals("3 to 4", range.getCoveredText());
		lowVal = range.getLowVal();
		highVal = range.getHighVal();
		assertTrue(
				lowVal instanceof de.unibi.techfak.isy.lppba.descriptors.Integer);
		assertTrue(
				highVal instanceof de.unibi.techfak.isy.lppba.descriptors.Integer);
		assertEquals(3,
				((de.unibi.techfak.isy.lppba.descriptors.Integer) lowVal).
				getValue());
		assertEquals(4,
				((de.unibi.techfak.isy.lppba.descriptors.Integer) highVal).
				getValue());

		assertTrue(iterator.hasNext());
		range = iterator.next();
		assertEquals("2.1- 3.5", range.getCoveredText());
		lowVal = range.getLowVal();
		highVal = range.getHighVal();
		assertTrue(
				lowVal instanceof de.unibi.techfak.isy.lppba.descriptors.Double);
		assertTrue(
				highVal instanceof de.unibi.techfak.isy.lppba.descriptors.Double);
		assertEquals(2.1,
				((de.unibi.techfak.isy.lppba.descriptors.Double) lowVal).
				getValue(), 0);
		assertEquals(3.5,
				((de.unibi.techfak.isy.lppba.descriptors.Double) highVal).
				getValue(), 0);

		assertTrue(iterator.hasNext());
		range = iterator.next();
		assertEquals("1- 3", range.getCoveredText());
		lowVal = range.getLowVal();
		highVal = range.getHighVal();
		assertTrue(
				lowVal instanceof de.unibi.techfak.isy.lppba.descriptors.Integer);
		assertTrue(
				highVal instanceof de.unibi.techfak.isy.lppba.descriptors.Integer);
		assertEquals(1,
				((de.unibi.techfak.isy.lppba.descriptors.Integer) lowVal).
				getValue());
		assertEquals(3,
				((de.unibi.techfak.isy.lppba.descriptors.Integer) highVal).
				getValue());

		assertFalse(iterator.hasNext());
	}
}
