/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.techfak.scie.annotators.structure;

import de.unibi.techfak.scie.annotators.structure.HeaderAnnotator;
import de.unibi.techfak.scie.annotators.ontology.OntologyAnnotator;
import de.unibi.techfak.scie.db.DatabaseSingleton;
import de.unibi.techfak.isy.lppba.descriptors.Header;
import de.unibi.techfak.scie.document_import.PDFStructuredTextExtractorWrapper;
import java.io.FileInputStream;
import java.util.Iterator;
import junit.framework.TestCase;
import org.apache.uima.fit.factory.JCasFactory;
import org.apache.uima.fit.util.JCasUtil;
import org.apache.uima.jcas.JCas;

/**
 *
 * @author Benjamin Paassen - bpaassen(at)techfak.uni-bielefeld.de
 */
public class HeaderAnnotatorTest extends TestCase {

	public HeaderAnnotatorTest(String testName) {
		super(testName);
	}

	@Override
	protected void setUp() throws Exception {
		super.setUp();
	}

	@Override
	protected void tearDown() throws Exception {
		super.tearDown();
	}

	/**
	 * Test of process method, of class HeaderAnnotator.
	 */
	public void testProcess() throws Exception {
		DatabaseSingleton.initialize("./config.xml");
		//parse a test document with the PDFStructuredTextExctractor first.
		final FileInputStream input = new FileInputStream("testdata/structure_test_document.pdf");
		final JCas jcas = JCasFactory.createJCas(
				"src/main/resources/de/unibi/techfak/isy/lppba/descriptors/LPPBATypeSystem");
		final PDFStructuredTextExtractorWrapper importer = new PDFStructuredTextExtractorWrapper(input, jcas);
		importer.doImport();
		//preprocess using OntologyAnnotator
		final OntologyAnnotator ontoAnno = new OntologyAnnotator();
		ontoAnno.process(jcas);
		//set up instance
		final HeaderAnnotator instance = new HeaderAnnotator();
		instance.process(jcas);
		//check results.
		final String[] expected = {"\t1 Introduction ", "\t2 Material and Methods ", "\t3 Results ",
			"\t4 Discussion "};
		final Iterator<Header> actual = JCasUtil.select(jcas, Header.class).iterator();
		for (final String exp : expected) {
			assertTrue(actual.hasNext());
			assertEquals(exp, actual.next().getCoveredText());
		}
	}
}
