/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.techfak.scie.annotators.treatment;

import de.unibi.techfak.scie.annotators.treatment.DoseAnnotator;
import java.util.Collection;
import java.util.Iterator;

import junit.framework.TestCase;

import org.apache.uima.UIMAException;
import org.apache.uima.fit.factory.JCasFactory;
import org.apache.uima.fit.util.JCasUtil;
import org.apache.uima.jcas.JCas;

import de.unibi.techfak.scie.annotators.quantities.NumberAnnotator;
import de.unibi.techfak.scie.annotators.quantities.RangeAnnotator;
import de.unibi.techfak.scie.annotators.quantities.WordNumberAnnotator;
import de.unibi.techfak.scie.annotators.structure.StanfordTokenizer;
import de.unibi.techfak.isy.lppba.descriptors.Dose;
;

public class DoseAnnotatorTest extends TestCase {

	public DoseAnnotatorTest(String testName) {
		super(testName);
	}

	@Override
	protected void setUp() throws Exception {
		super.setUp();
	}

	@Override
	protected void tearDown() throws Exception {
		super.tearDown();
	}


	/**
	 * Test of handleQuantity method, of class WeightAnnotator.
	 */
	public void testHandleQuantity() throws UIMAException {

		//create example
		final JCas jcas = JCasFactory.createJCas(
				"src/main/resources/de/unibi/techfak/isy/lppba/descriptors/LPPBATypeSystem");
		jcas.setDocumentText("100 iu/kg in 200mg/kg, 300 units/kg, 3 mg/kg/h and 2 unit/kg");
		//preprosess using StanfordTokenizer, NumberAnnotator, RangeAnnotator and WordNumberAnnotator
		final StanfordTokenizer st = new StanfordTokenizer();
		st.process(jcas);
		final NumberAnnotator numberAnnotator = new NumberAnnotator();
		numberAnnotator.process(jcas);
		final WordNumberAnnotator wordNumberAnnotator = new WordNumberAnnotator();
		wordNumberAnnotator.process(jcas);
		final RangeAnnotator rangeAnnotator = new RangeAnnotator();
		rangeAnnotator.process(jcas);

		//use the DoseAnnotator
		final DoseAnnotator instance = new DoseAnnotator();
		instance.process(jcas);

		//check results
		final Collection<Dose> DoseInstances = JCasUtil.select(jcas, Dose.class);

		final Iterator<Dose> iterator = DoseInstances.iterator();
		assertTrue(iterator.hasNext());
		Dose dose = iterator.next();
		assertEquals("100 iu/kg", dose.getCoveredText());
		assertEquals("100", dose.getValue().getCoveredText());
		assertEquals("international unit/Kilogrammes", dose.getUnitName());

		assertTrue(iterator.hasNext());
		dose = iterator.next();
		assertEquals("200mg/kg", dose.getCoveredText());
		assertEquals("200", dose.getValue().getCoveredText());
		assertEquals("Milligrammes/Kilogrammes", dose.getUnitName());
		
		assertTrue(iterator.hasNext());
		dose = iterator.next();
		assertEquals("300 units/kg", dose.getCoveredText());
		assertEquals("300", dose.getValue().getCoveredText());
		assertEquals("international unit/Kilogrammes", dose.getUnitName());
		
		assertTrue(iterator.hasNext());
		dose = iterator.next();
		assertEquals("3 mg/kg/h", dose.getCoveredText());
		assertEquals("3", dose.getValue().getCoveredText());
		assertEquals("Milligrammes/Kilogrammes/Hours", dose.getUnitName());
		
		assertTrue(iterator.hasNext());
		dose = iterator.next();
		assertEquals("2 unit/kg", dose.getCoveredText());
		assertEquals("2", dose.getValue().getCoveredText());
		assertEquals("international unit/Kilogrammes", dose.getUnitName());

		assertFalse(iterator.hasNext());
		
		}
}
