/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibi.techfak.scie.document_import;

import de.unibi.techfak.scie.document_import.PDFStructuredTextExtractorWrapper;
import de.unibi.techfak.isy.lppba.descriptors.DocumentPage;
import de.unibi.techfak.isy.lppba.descriptors.Paragraph;
import de.unibi.techfak.isy.lppba.descriptors.TextBlock;
import de.unibi.techfak.scie.util.CachedJCasUtil;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;

import junit.framework.TestCase;
import org.apache.uima.UIMAException;

import org.apache.uima.fit.factory.JCasFactory;
import org.apache.uima.fit.util.JCasUtil;
import org.apache.uima.jcas.JCas;

/**
 *
 * @author Benjamin Paassen - bpaassen(at)techfak.uni-bielefeld.de
 */
public class PDFStructuredTextExtractorTest extends TestCase {

	public PDFStructuredTextExtractorTest(String testName) {
		super(testName);
	}

	@Override
	protected void setUp() throws Exception {
		super.setUp();
	}

	@Override
	protected void tearDown() throws Exception {
		super.tearDown();
	}

	/**
	 * Test of doImport method, of class PDFStructuredTextExtractor. We do a
	 * heuristic test if the importer does import something at all.
	 */
	public void testDoImport() throws Exception {

		// Iterate over all ".pdf" files in the testdata folder and sub folders
		File folder = new File("./testdata");
		File[] files = folder.listFiles();
		for (File file : files) {
			if (file.getName().endsWith(".pdf")) {
				System.out.println("Processing file " + file.getName());
				checkPDF(file);
			}
			if (file.isDirectory()) {
				File[] files2 = file.listFiles();
				for (File file2 : files2) {
					System.out.println("Processing file " + file2.getName());
					if (file2.getName().endsWith(".pdf")) {
						checkPDF(file2);
					}
				}
			}
		}
		assertEquals("This test should not fail.", true, true);
	}

	private static void checkPDF(final File pdf) throws IOException, UIMAException {
		final JCas jcas = JCasFactory.createJCas(
				"src/main/resources/de/unibi/techfak/isy/lppba/descriptors/LPPBATypeSystem");
		final FileInputStream input = new FileInputStream(pdf);
		//import pdf
		final PDFStructuredTextExtractorWrapper instance = new PDFStructuredTextExtractorWrapper(input, jcas);

		instance.doImport();

		//check result.
		final CachedJCasUtil jcasUtil = CachedJCasUtil.getInstance(jcas);
		//we expect at least one page.
		final Collection<DocumentPage> pages = JCasUtil.select(jcas, DocumentPage.class);
		assertFalse(pages.isEmpty());
		//we expect that the TextBlocks in the document are equal to the TextBlocks covered by
		//all pages
		final Collection<TextBlock> expectedTextBlocks = JCasUtil.select(jcas, TextBlock.class);
		assertFalse(expectedTextBlocks.isEmpty());
		final ArrayList<TextBlock> actualTextBlocks = new ArrayList<>();
		for (final DocumentPage page : pages) {
			actualTextBlocks.addAll(jcasUtil.selectCovered(TextBlock.class,
					page.getBegin(), page.getEnd()));
		}
		assertEquals(expectedTextBlocks.size(), actualTextBlocks.size());
		int i = 0;
		for (final TextBlock exp : expectedTextBlocks) {
			assertSame(exp, actualTextBlocks.get(i));
			i++;
		}

		//we expect that the Paragraphs in the document are equal to the Paragraphs covered by all
		//text blocks.
		final Collection<Paragraph> expectedParagraphs = JCasUtil.select(jcas, Paragraph.class);
		assertFalse(expectedParagraphs.isEmpty());
		final ArrayList<Paragraph> actualParagraphs = new ArrayList<>();
		for (final TextBlock block : expectedTextBlocks) {
			actualParagraphs.addAll(jcasUtil.selectCovered(Paragraph.class,
					block.getBegin(), block.getEnd()));
		}
		assertEquals(expectedParagraphs.size(), actualParagraphs.size());
		i = 0;
		for (final Paragraph exp : expectedParagraphs) {
			assertSame(exp, actualParagraphs.get(i));
			i++;
		}
	}
}
