/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unibi.techfak.scie.util;

import de.unibi.techfak.scie.util.LineSegment;
import junit.framework.TestCase;

/**
 *
 * @author Andreas Stoeckel - astoecke(at)techfak.uni-bielefeld.de
 */
public class LineSegmentTest extends TestCase {
	
	public LineSegmentTest(String testName) {
		super(testName);
	}

	/**
	 * Test of overlaps method, of class LineSegment.
	 */
	public void testOverlaps() {
		LineSegment l1 = new LineSegment();
		LineSegment l2 = new LineSegment();

		l1.setBegin(0);
		l1.setEnd(10);
		l2.setBegin(5);
		l2.setEnd(15);
		assertEquals(true, LineSegment.overlaps(l1, l2));

		l2.setBegin(0);
		l2.setEnd(10);
		l1.setBegin(5);
		l1.setEnd(15);
		assertEquals(true, LineSegment.overlaps(l1, l2));

		l2.setBegin(0);
		l2.setEnd(10);
		l1.setBegin(10);
		l1.setEnd(20);
		assertEquals(true, LineSegment.overlaps(l1, l2));

		l2.setBegin(0);
		l2.setEnd(10);
		l1.setBegin(11);
		l1.setEnd(21);
		assertEquals(false, LineSegment.overlaps(l1, l2));

		l2.setBegin(5);
		l2.setEnd(10);
		l1.setBegin(0);
		l1.setEnd(20);
		assertEquals(true, LineSegment.overlaps(l1, l2));

		l1.setBegin(5);
		l1.setEnd(10);
		l2.setBegin(0);
		l2.setEnd(20);
		assertEquals(true, LineSegment.overlaps(l1, l2));

		l1.setBegin(5);
		l1.setEnd(10);
		l2.setBegin(20);
		l2.setEnd(0);
		assertEquals(true, LineSegment.overlaps(l1, l2));

		l1.setBegin(10);
		l1.setEnd(5);
		l2.setBegin(20);
		l2.setEnd(0);
		assertEquals(true, LineSegment.overlaps(l1, l2));

		l1.setBegin(10);
		l1.setEnd(5);
		l2.setBegin(0);
		l2.setEnd(20);
		assertEquals(true, LineSegment.overlaps(l1, l2));
	}

	/**
	 * Test of union method, of class LineSegment.
	 */
	public void testUnion() {
		LineSegment l1 = new LineSegment();
		LineSegment l2 = new LineSegment();
		LineSegment l;

		l1.setBegin(0);
		l1.setEnd(10);
		l2.setBegin(5);
		l2.setEnd(15);

		l = LineSegment.union(l1, l2);
		assertEquals(0, l.begin);
		assertEquals(15, l.end);

		l1.setBegin(0);
		l1.setEnd(10);
		l2.setBegin(100);
		l2.setEnd(15);

		l = LineSegment.union(l1, l2);
		assertEquals(0, l.begin);
		assertEquals(100, l.end);
	}

	/**
	 * Test of union method, of class LineSegment.
	 */
	public void testIntersection() {
		LineSegment l1 = new LineSegment();
		LineSegment l2 = new LineSegment();
		LineSegment l;

		l1.setBegin(0);
		l1.setEnd(10);
		l2.setBegin(5);
		l2.setEnd(15);

		l = LineSegment.intersection(l1, l2);
		assertEquals(5, l.begin);
		assertEquals(10, l.end);

		l1.setBegin(0);
		l1.setEnd(10);
		l2.setBegin(100);
		l2.setEnd(15);

		l = LineSegment.intersection(l1, l2);
		assertEquals(null, l);
	}


	
}
