/*
 * SCIE -- Spinal Cord Injury Ontology Extraction
 * Copyright (C) 2013, 2014
 * Raphael Dickfelder, Jan Göpfert, Benjamin Paaßen, Andreas Stöckel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibi.techfak.scie;

import java.util.ArrayList;
import org.apache.uima.analysis_engine.AnalysisEngineProcessException;
import org.apache.uima.fit.component.JCasAnnotator_ImplBase;
import org.apache.uima.jcas.JCas;

/**
 * This is a simple pipeline class modelling the iterative application of
 * different annotators.
 *
 * TODO: Ideally this should be removed if we are able to exploit the UIMA
 * functionality of parallel processing.
 *
 * @author Benjamin Paassen - bpaassen@techfak.uni-bielefeld.de
 */
public class AnnotatorPipeline {
    
    private final ArrayList<JCasAnnotator_ImplBase> annotators = new ArrayList<>();
    
    public AnnotatorPipeline() {
    }
    
    public ArrayList<JCasAnnotator_ImplBase> getAnnotators() {
        return annotators;
    }

    /**
     * Runs all annotators stored in this pipeline on the given input JCas
     * instance. The results are stored in the given JCas instance as well.
     *
     * @param input a JCas instance.
     * @throws AnalysisEngineProcessException may be thrown by an annotator.
     */
    public void process(JCas input) throws AnalysisEngineProcessException {
        for (final JCasAnnotator_ImplBase annotator : annotators) {
            annotator.process(input);
        }
    }
    
}
